<?php
/**
 * Legacy Functions
 *
 * WC_GZD legacy functions.
 *
 * @author 		Vendidero
 * @version     1.0.0
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

function wc_gzd_get_crud_data( $object, $key ) {

	if ( is_a( $object, 'WC_GZD_Product' ) ) {
		$object = $object->get_wc_product();
	}

	$value = null;

	$getter = substr( $key, 0, 3 ) === "get" ? $key : "get_$key";
	$key = substr( $key, 0, 3 ) === "get" ? substr( $key, 3 ) : $key;

	if ( 'id' === $key && is_callable( array( $object, 'is_type' ) ) && $object->is_type( 'variation' ) && ! WC_GZD_Dependencies::instance()->woocommerce_version_supports_crud() ) {
		$key = 'variation_id';
	} else if ( 'parent' === $key && is_callable( array( $object, 'is_type' ) ) && $object->is_type( 'variation' ) && ! WC_GZD_Dependencies::instance()->woocommerce_version_supports_crud() ) {
	    // Set getter to parent so that it is not being used for pre 2.7
	    $key = 'id';
	    $getter = 'parent';
    }

	$getter_mapping = array(
		'parent' => 'get_parent_id',
		'completed_date' => 'get_date_completed',
		'order_date' => 'get_date_created',
		'product_type' => 'get_type',
		'order_type' => 'get_type',
	);

	if ( array_key_exists( $key, $getter_mapping ) ) {
		$getter = $getter_mapping[ $key ];
	}

	if ( is_callable( array( $object, $getter ) ) ) {
		$reflection = new ReflectionMethod( $object, $getter );
		if ( $reflection->isPublic() ) {
			$value = $object->{$getter}();
		}
	} else if ( WC_GZD_Dependencies::instance()->woocommerce_version_supports_crud() ) {
		if ( substr( $key, 0, 1 ) !== '_' )
			$key = '_' . $key;

		$value = $object->get_meta( $key );
	} else {
		$key = substr( $key, 0, 1 ) === "_" ? substr( $key, 1 ) : $key;
		$value = $object->{$key};
	}

	return $value;
}

function wc_gzd_get_variable_visible_children( $product ) {
	if ( is_callable( array( $product, 'get_visible_children' ) ) )
		return $product->get_visible_children();
	return $product->get_children( true );
}

function wc_gzd_get_price_including_tax( $product, $args = array() ) {
	if ( function_exists( 'wc_get_price_including_tax' ) )
		return wc_get_price_including_tax( $product, $args );
	return $product->get_price_including_tax( $args[ 'qty' ], $args[ 'price' ] );
}

function wc_gzd_get_price_excluding_tax( $product, $args = array() ) {
	if ( function_exists( 'wc_get_price_excluding_tax' ) )
		return wc_get_price_excluding_tax( $product, $args );
	return $product->get_price_excluding_tax( $args[ 'qty' ], $args[ 'price' ] );
}

function wc_gzd_get_variation( $parent, $variation ) {
	if ( WC_GZD_Dependencies::instance()->woocommerce_version_supports_crud() )
		return wc_get_product( $variation );
	return $parent->get_child( $variation );
}

function wc_gzd_get_order_currency( $order ) {
	if ( WC_GZD_Dependencies::instance()->woocommerce_version_supports_crud() )
		return $order->get_currency();
	return $order->get_order_currency();
}