<?php
/**
 * Product Functions
 *
 * WC_GZD product functions.
 *
 * @author 		Vendidero
 * @version     1.0.0
 */

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/**
 * Register unit price update hook while cronjob is running
 */
function wc_gzd_register_scheduled_unit_sales() {
	add_action( 'updated_post_meta', 'wc_gzd_check_price_update', 0, 4 );
}
add_action( 'woocommerce_scheduled_sales', 'wc_gzd_register_scheduled_unit_sales', 0 );

function wc_gzd_get_gzd_product( $product ) {
	
	if ( ! isset( $product->gzd_product ) || ! is_object( $product->gzd_product ) )
		$product->gzd_product = WC()->product_factory->get_gzd_product( $product );

	return $product->gzd_product;
}

/**
 * Unregister unit price update hook
 */
function wc_gzd_unregister_scheduled_unit_sales() {
	remove_action( 'updated_post_meta', 'wc_gzd_check_price_update', 0 );
}
add_action( 'woocommerce_scheduled_sales', 'wc_gzd_unregister_scheduled_unit_sales', 20 );

/**
 * Update the unit price to sale price if product is on sale
 */
function wc_gzd_check_price_update( $meta_id, $post_id, $meta_key, $meta_value ) {

	if ( $meta_key != '_price' )
		return;

	$product = wc_get_product( $post_id );
	$sale_price = get_post_meta( $post_id, '_unit_price_sale', true );
	$regular_price = get_post_meta( $post_id, '_unit_price_regular', true );
	
	if ( $product->is_on_sale() && $sale_price ) {
		update_post_meta( $post_id, '_unit_price', $sale_price );
	} else {
		update_post_meta( $post_id, '_unit_price', $regular_price );
	}

}

function wc_gzd_get_small_business_product_notice() {
	return apply_filters( 'woocommerce_gzd_small_business_product_notice', __( 'VAT free based on &#167;19 UStG', 'woocommerce-germanized' ) );
}

function wc_gzd_is_revocation_exempt( $product, $type = 'digital' ) {
	
	$types = apply_filters( 'woocommerce_gzd_digital_product_types', get_option( 'woocommerce_gzd_checkout_legal_digital_types', array( 'downloadable' ) ) );

	if ( empty( $types ) )
		return false;
	
	else if ( ! is_array( $types ) )
		$types = array( $types );

	if ( $type === 'digital' ) {
		foreach ( $types as $revo_type ) {

			if ( wc_gzd_product_matches_extended_type( $revo_type, $product ) )
				return true;
		}
	} else if ( $type === 'service' ) {
		if ( wc_gzd_get_gzd_product( $product )->is_service() )
			return true;
	}

	if ( apply_filters( 'woocommerce_gzd_product_is_revocation_exception', false, $product, $type ) )
		return true;

	return false;
}

function wc_gzd_product_matches_extended_type( $type, $product ) {
	
	$parent = null;

	// Support Variations
	if ( wc_gzd_get_crud_data( $product, 'parent' ) ) {
		$parent = wc_get_product( wc_gzd_get_crud_data( $product, 'parent' ) );
	}

	$matches_type = false;

	if ( $type === $product->get_type() )
		$matches_type = true; 
	else if ( is_callable( array( $product, 'is_' . $type ) ) )
		$matches_type = ( call_user_func_array( array( $product, 'is_' . $type ), array() ) === true ? true : false );
	
	// Do only check parent if product does not match the type
	if ( ! $matches_type && $parent )
		return wc_gzd_product_matches_extended_type( $type, $parent );

	return $matches_type;
}